/* File    - menu.c
 * Author  - Jared Eby
 * Date    - 1/10/2016
 * Purpose - Gets data from the UART and uses it to traverse the menu structures.
 *           The current menu pointer is updated and functions are called when 
 *           appropriate.
 */

#include <stdio.h>
#include "PSoCAPI.h"
#include "menu.h"
#include "menu_struct.h"



/*This function uses user input to navigate the menu
 */
void menu_process (void){
	char input;
	short i;

	//while (!DataRdyUSART());
	input = UART_cGetChar();		

	/*Returns to the previous tier of menu
 	 */
	if (input == 'R' || input == 'r') {
		
		UART_PutChar(input);
		UART_CPutString((const char*) " -------> Press 'Enter' to return to the previous menu.");
		promptEnter( &input);

		current_menu = current_menu->prevmenu;
		printMenu();


	/*Returns to the top tier of menu
	 */
	} else if (input == 'T' || input == 't') {

		UART_PutChar(input);
		UART_CPutString((const char*) " -------> Press 'Enter' to return to the main menu.");
		promptEnter( &input);

		current_menu = current_menu->topmenu;
		printMenu();


	/*Exits the menu
	 */
	} else if (input == 'X' || input == 'x') {

		UART_PutChar(input);
		UART_CPutString((const char*) " -------> Press 'Enter' to return to exit the menu.");
		promptEnter( &input);
		UART_PutChar(12); 			//Clear the screen in Hyper terminal window

		UART_CPutString((const char*) "The menu has been shut down.  Press 'X' again to restart.");
		while (input != 0x58 && input != 0x78) {
			//while (!DataRdyUSART());
			input = UART_cGetChar();
		}

		UART_PutChar(12); 			//Clear the screen in Hyper terminal window
		printMenu();


	/*Enters corresponding sub menu or executes corresponding code when appropriate
	 */
	} else if (input == '1' || input == '2' || input == '3') {
		i = atoi(&input);

		if (current_menu->menusize >= i) {
			
			UART_PutChar(input);
			UART_CPutString((const char*) " -------> Press 'Enter' to select this item.");
			promptEnter( &input);
	
			if (current_menu->menulist[i-1].itemtype == 'M') {
				current_menu = current_menu->menulist[i-1].item;
			} else if (current_menu->menulist[i-1].itemtype == 'F') {
				UART_PutChar(12); 			//Clear the screen in Hyper terminal window
//				((void(*)(void))(current_menu->menulist[i-1].item))();
//				((void*)current_menu->menulist[i-1].item)();
				UART_CPutString((const char*) "\n\n\rPress 'Enter' to return to the menu.\n\n\r");
				input = '0';
				promptEnter(&input);
			}

			printMenu();
		}


	/*Prints a message if the user input isn't valid.
	 */
	} else {

		UART_CPutString((const char*) "\n\n\r -------> This is not a valid character.\n\r");


	}
}


/*Clears the screen and prints out the current menu
 */
void printMenu (void) {
	short i;
	
	UART_PutChar(12); 			//Clear the screen in Hyper terminal window
	UART_CPutString((const char*)current_menu->heading);
	UART_CPutString((const char*) "\n\r");
	
	for (i = 0; i < current_menu->menusize; i++) {
		UART_CPutString((const char*) "\n\r\t");
		UART_PutChar(current_menu->menulist[i].id);
		UART_CPutString((const char*) ".  ");
		UART_CPutString((const char*) current_menu->menulist[i].text);
	}

	UART_CPutString((const char*) "\n\n\rR. Return        T. Top        X. Exit\n\n\r");
	//test_func1();
	(*fptr)();
}


/*Blocks further code execution until the user hits 'Enter'.
 */
void promptEnter (char *input) {
	while (*input != 0xD) {
		//while (!DataRdyUSART());
		*input = UART_cGetChar();
	}
}