//-----------------------------------------------------------------------
// This file creates the API for interfacing with a two-digit Nixie 
// module via the module's shift registers.
//-----------------------------------------------------------------------
#include "tubecntrl.h"		// API for loading numbers onto a Nixie display module
#include "PSoCAPI.h"    	// PSoC API definitions for all User Modules


//Define the number of shifts needed to display each number. The first dimension
//specifies the digit, while the second is for the number of shifts to display the
//index.
char tubeSec[2][10] = {{7,0,1,2,3,10,11,4,5,6},{13,14,15,8,9,16,17,18,19,12}};


//This function is meant to be called elsewhere and displays the number passed.
void updateNixieDisplay(char num, char colonFlag) {
	char msNum, lsNum, shift1, shift2, i, j;
	
	//Calculate digits
	lsNum = num&0x0f;
	msNum = num>>4;
	   
	//Calculate # of shifts to display digits
	if (num == 0x53 || num == 0x54 || num == 0x63 || num == 0x64) {
		shift1 = tubeSec[0][msNum]-(tubeSec[1][lsNum]+1);
		shift2 = tubeSec[1][lsNum];
	} else {
		shift1 = tubeSec[1][lsNum]-(tubeSec[0][msNum]+1);
		shift2 = tubeSec[0][msNum];
	}
	   
	//Control the shift registers
	clearNixieDisplay();		//Clear the module
	loadBit();			//Load a '1' into the shift registers
	shiftBit(shift1);	//Shift the '1'
	loadBit();			//Load another '1' into the shift registers
	shiftBit(shift2);	//Shift the contents
}


//This function clears the digits from the Nixie module
void clearNixieDisplay(void) {
	char i;
	
	//Make sure the data and clock are at 0, then cycle it until nothing is displayed
	Port_4_Data_SHADE |= 0X08;		//Set STROBE high (0X01)
	PRT4DR = Port_4_Data_SHADE;
	PRT3DR &= ~0X80;				//set data to 0
	Port_4_Data_SHADE &= ~0X20;		//set clock to 0
	PRT4DR = Port_4_Data_SHADE;
	for (i=0;i<50;i++) {
		PRT4DR |= 0X20;				//set clock to 1
		PRT4DR = Port_4_Data_SHADE;
		PRT4DR &= ~0X20;			//set clock to 0
		PRT4DR = Port_4_Data_SHADE;
	}
}


//This function loads a '1' into the first slot on the first shift register
void loadBit(void) {
	//0X02 = CLK and 0x04 = DATA
	
	PRT3DR |= 0X80;					//set data to 1
	Port_4_Data_SHADE |= 0X20;		//set clock to 1
	PRT4DR = Port_4_Data_SHADE;
	Port_4_Data_SHADE &= ~0X20;		//set clock to 0
	PRT4DR = Port_4_Data_SHADE;
	PRT3DR &= ~0X80;				//set data to 0
}


//This function shifts the contents of the shift registers by a specified amount.
//Care must be taken to ensure the data line is in the correct position before using
//this function.
void shiftBit(char shift) {
	char i;
	
	//Cycle the clock the specified number of times
	for (i=0;i<shift;i++) {
		Port_4_Data_SHADE |= 0X20;		//set clock to 1
		PRT4DR = Port_4_Data_SHADE;
		Port_4_Data_SHADE &= ~0X20;		//set clock to 0
		PRT4DR = Port_4_Data_SHADE;
	}
}
